@@ -25,7 +25,7 @@ import yaml
 
 from pyxllib.algo.pupil import Groups
 from pyxllib.file.pupil import struct_unpack, gen_file_filter
-from pyxllib.prog.pupil import is_url, is_file
+from pyxllib.prog.pupil import is_url, is_file, DictTool
 
 ____judge = """
 """
@@ -86,7 +86,7 @@ def test_etag2():
     print(get_etag(s))
     # FkAD2McB6ugxTiniE8ebhlNHdHh9
 
-    f = File('1.tex', tempfile.gettempdir()).write(s, if_exists='delete').to_str()
+    f = File('1.tex', tempfile.gettempdir()).write(s, if_exists='replace').to_str()
     print(get_etag(f))
     # FkAD2McB6ugxTiniE8ebhlNHdHh9
 
@@ -207,7 +207,7 @@ class PathBase:
     __slots__ = ('_path',)
 
     @classmethod
-    def abspath(cls, path=None, root=None, *, suffix=None) -> pathlib.Path:
+    def abspath(cls, path=None, root=None, *, suffix=None, resolve=True) -> pathlib.Path:
         r""" 根据各种不同的组合参数信息，推导出具体的路径位置
 
         :param path: 主要的参数，如果后面的参数有矛盾，以path为最高参考标准
@@ -271,7 +271,10 @@ class PathBase:
             elif li[1] == '':
                 path = li[0] + '.' + suffix
 
-        return pathlib.Path(path).resolve()
+        if resolve:
+            return pathlib.Path(path).resolve()
+        else:
+            return pathlib.Path(path)
 
     def __bool__(self):
         r""" 判断文件、文件夹是否存在
@@ -423,7 +426,10 @@ class PathBase:
         :param if_exists:
             None: 不做任何处理，直接运行，依赖于功能本身是否有覆盖写入机制
             'error': 如果要替换的目标文件已经存在，则报错
-            'delete': 把存在的文件先删除
+            'replace': 把存在的文件先删除
+                本来是叫'delete'更准确的，但是考虑用户理解，
+                    一般都是用在文件替换场合，叫成'delete'会非常怪异，带来不必要的困扰、误解
+                所以还是决定叫'replace'
             'skip': 不执行后续功能
             'backup': 先做备份  （对原文件先做一个备份）
         """
@@ -433,7 +439,7 @@ class PathBase:
                 return need_run
             elif if_exists == 'error':
                 raise FileExistsError(f'目标文件已存在： {self}')
-            elif if_exists == 'delete':
+            elif if_exists == 'replace':
                 self.delete()
             elif if_exists == 'skip':
                 need_run = False
@@ -452,7 +458,7 @@ class PathBase:
     def absdst(self, dst):
         raise NotImplementedError
 
-    def backup(self, tail=None, if_exists='delete', move=False):
+    def backup(self, tail=None, if_exists='replace', move=False):
         r""" 对文件末尾添加时间戳备份，也可以使用自定义标记tail
 
         :param tail: 自定义添加后缀
@@ -535,14 +541,21 @@ class File(PathBase):
 
     # 一、基础功能
 
-    def __init__(self, path, root=None, *, suffix=None):
+    def __init__(self, path, root=None, *, suffix=None, check=True):
         r""" 初始化参数含义详见 PathBase.abspath 函数解释
 
+        :param path: 只传入一个File、pathlib.Path对象，可以提高初始化速度，不会进行多余的解析判断
+            如果要安全保守一点，可以传入str类型的path
+        :param root: 父目录
+        :param check: 在某些特殊场合、内部开发中，可以确保传参一定不会出错，在上游已经有了严谨的检查
+            此时可以设置check=False，提高初始化速度
+
         注意！如果使用了符号链接（软链接），则路径是会解析转向实际位置的！例如
         >> File('D:/pycode/code4101py')
         File('D:/slns/pycode/code4101py')
         """
         self._path = None
+
         # 1 快速初始化
         if root is None and suffix is None:
             if isinstance(path, File):
@@ -550,13 +563,17 @@ class File(PathBase):
                 return  # 直接完成初始化过程
             elif isinstance(path, pathlib.Path):
                 self._path = path
+
         # 2 普通初始化
         if self._path is None:
             self._path = self.abspath(path, root, suffix=suffix)
-        if not self._path:
-            raise ValueError(f'无效路径 {self._path}')
-        elif self._path.is_dir():
-            raise ValueError(f'不能用目录初始化一个File对象 {self._path}')
+
+        # 3 检查
+        if check:
+            if not self._path:
+                raise ValueError(f'无效路径 {self._path}')
+            elif self._path.is_dir():
+                raise ValueError(f'不能用目录初始化一个File对象 {self._path}')
 
     @classmethod
     def safe_init(cls, path, root=None, *, suffix=None):
@@ -656,8 +673,9 @@ class File(PathBase):
     def absdst(self, dst):
         """ 在copy、move等中，给了个"模糊"的目标位置dst，智能推导出实际file、dir绝对路径
         """
+        from pyxllib.file.specialist.dirlib import Dir
         dst_ = self.abspath(dst)
-        if (isinstance(dst, str) and dst[-1] in ('\\', '/')) or dst_.is_dir():
+        if isinstance(dst, Dir) or (isinstance(dst, str) and dst[-1] in ('\\', '/')) or dst_.is_dir():
             dst_ = File(self.name, dst_)
         else:
             dst_ = File(dst_)
@@ -769,8 +787,7 @@ class File(PathBase):
                     pickle.dump(ob, f)
             elif mode == '.json':
                 with open(name, 'w', encoding=encoding) as f:
-                    if 'ensure_ascii' not in kwargs:
-                        kwargs['ensure_ascii'] = False
+                    DictTool.ior(kwargs, {'ensure_ascii': False})
                     ujson.dump(ob, f, **kwargs)
             elif mode == '.yaml':
                 with open(name, 'w', encoding=encoding) as f:
@@ -999,7 +1016,7 @@ class UsedRecords:
     def save(self):
         """保存记录文件"""
         File(self.dirname + '/').ensure_parent()
-        File(self.fullname).write('\n'.join(self.ls), if_exists='delete')
+        File(self.fullname).write('\n'.join(self.ls), if_exists='replace')
 
     def add(self, s):
         """新增一个使用方法
